"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var ReactDOM = require("react-dom");
var prefix_1 = require("@devexperts/utils/dist/dom/prefix");
var classnames = require("classnames");
var pure_1 = require("../../utils/pure");
var BoundsUpdateDetector_1 = require("../BoundsUpdateDetector/BoundsUpdateDetector");
var throttle_1 = require("@devexperts/utils/dist/function/throttle");
var withTheme_1 = require("../../utils/withTheme");
var EventListener_1 = require("../EventListener/EventListener");
var RootClose_1 = require("../RootClose/RootClose");
var react_dom_1 = require("react-dom");
exports.POPOVER = Symbol('Popover');
var PopoverPlacement;
(function (PopoverPlacement) {
    PopoverPlacement["Top"] = "Top";
    PopoverPlacement["Bottom"] = "Bottom";
    PopoverPlacement["Left"] = "Left";
    PopoverPlacement["Right"] = "Right";
})(PopoverPlacement = exports.PopoverPlacement || (exports.PopoverPlacement = {}));
var PopoverAlign;
(function (PopoverAlign) {
    PopoverAlign["Top"] = "Top";
    PopoverAlign["Bottom"] = "Bottom";
    PopoverAlign["Left"] = "Left";
    PopoverAlign["Right"] = "Right";
    PopoverAlign["Middle"] = "Middle";
    PopoverAlign["Center"] = "Center";
})(PopoverAlign = exports.PopoverAlign || (exports.PopoverAlign = {}));
var RawPopover = /** @class */ (function (_super) {
    tslib_1.__extends(RawPopover, _super);
    function RawPopover(props) {
        var _this = _super.call(this, props) || this;
        _this.state = {};
        _this._needsUpdate = false;
        _this.onSizeUpdate = function (newSize) {
            _this._popoverSize = newSize;
            _this.updatePosition();
        };
        _this.onResize = function () {
            _this.handleResize();
        };
        _this.onScroll = function () {
            _this.handleScroll();
        };
        _this.rootElement = document.createElement('div');
        var container = props.container || document.body;
        container.appendChild(_this.rootElement);
        return _this;
    }
    RawPopover.prototype.componentDidMount = function () {
        if (this.props.isOpened) {
            if (this.props.anchor) {
                var anchorDOMNode = ReactDOM.findDOMNode(this.props.anchor);
                if (!anchorDOMNode || anchorDOMNode instanceof Text) {
                    return;
                }
                this._anchor = anchorDOMNode;
            }
            this._popoverSize = this.getPopoverSize();
            this.updatePosition();
        }
    };
    RawPopover.prototype.componentWillUnmount = function () {
        var container = this.props.container || document.body;
        container.removeChild(this.rootElement);
    };
    RawPopover.prototype.componentWillReceiveProps = function (nextProps) {
        if (nextProps.isOpened && nextProps.anchor) {
            this._needsUpdate = true;
            var anchorDOMNode = ReactDOM.findDOMNode(nextProps.anchor);
            if (!anchorDOMNode || anchorDOMNode instanceof Text) {
                return;
            }
            this._anchor = anchorDOMNode;
        }
    };
    RawPopover.prototype.componentDidUpdate = function (prevProps) {
        //@PURE check is passed here - update anyway
        if (this._needsUpdate) {
            /** set in {@link componentWillReceiveProps} */
            this._needsUpdate = false;
            this._popoverSize = this.getPopoverSize();
            this.updatePosition();
        }
    };
    RawPopover.prototype.render = function () {
        var _this = this;
        var _a = this.props, closeOnClickAway = _a.closeOnClickAway, theme = _a.theme, hasArrow = _a.hasArrow, onMouseDown = _a.onMouseDown, isOpened = _a.isOpened, onRequestClose = _a.onRequestClose, anchor = _a.anchor;
        if (!isOpened || !anchor) {
            return null;
        }
        var _b = this.state, top = _b.top, left = _b.left, arrowOffset = _b.arrowOffset, finalPlacement = _b.finalPlacement, finalAlign = _b.finalAlign;
        var isMeasured = !!finalAlign && !!finalPlacement;
        var style;
        var popoverClassName = classnames(theme.container);
        if (isMeasured && finalPlacement) {
            style = prefix_1.default({
                transform: "translate(" + (left || 0) + "px, " + (top || 0) + "px)",
            });
            popoverClassName = classnames(popoverClassName, (_c = {},
                _c[theme.container_hasArrow] = hasArrow,
                _c), [getPlacementModifier(finalPlacement)].map(function (mod) { return theme["container_" + mod]; }));
        }
        style = tslib_1.__assign({}, style, this.props.style);
        var child = (React.createElement(BoundsUpdateDetector_1.BoundsUpdateDetector, { onUpdate: this.onSizeUpdate },
            React.createElement("div", { ref: function (el) { return (_this._popover = el); }, style: style, onMouseDown: onMouseDown, className: popoverClassName },
                React.createElement("div", { className: theme.content },
                    isMeasured &&
                        finalPlacement &&
                        finalAlign &&
                        hasArrow && (React.createElement("div", { className: theme.arrow, style: getArrowStyle(finalPlacement, finalAlign, arrowOffset) })),
                    this.props.children))));
        if (closeOnClickAway) {
            child = React.createElement(RootClose_1.RootClose, { onRootClose: onRequestClose }, child);
        }
        child = (React.createElement(EventListener_1.EventListener, { onResize: this.onResize, onScroll: this.onScroll, target: "window" }, child));
        return react_dom_1.createPortal(child, this.rootElement);
        var _c;
    };
    RawPopover.prototype.getPopoverSize = function () {
        var popover = ReactDOM.findDOMNode(this._popover);
        return {
            height: popover.offsetHeight,
            width: popover.offsetWidth,
        };
    };
    RawPopover.prototype.updatePosition = function () {
        if (!this._anchor) {
            return;
        }
        var anchorRect = this._anchor.getBoundingClientRect();
        var _a = this.props, placement = _a.placement, align = _a.align, hasArrow = _a.hasArrow;
        var arrowOffset;
        var finalPlacement;
        var finalAlign;
        var topResult = movePopoverVertically(placement, align, anchorRect.top, anchorRect.bottom, this._popoverSize.height, true);
        var leftResult = movePopoverHorizontally(placement, align, anchorRect.left, anchorRect.right, this._popoverSize.width, true);
        //additional
        if (placement === PopoverPlacement.Top || placement === PopoverPlacement.Bottom) {
            finalPlacement = topResult.placement;
            finalAlign = leftResult.align;
            if (hasArrow) {
                //horizontal arrow offset
                arrowOffset = Math.round((anchorRect.right - anchorRect.left) / 2);
            }
        }
        else if (placement === PopoverPlacement.Left || placement === PopoverPlacement.Right) {
            finalPlacement = leftResult.placement;
            finalAlign = topResult.align;
            if (hasArrow) {
                //vertical arrow offset
                arrowOffset = Math.round((anchorRect.bottom - anchorRect.top) / 2);
            }
        }
        this.setState({
            top: Math.round(topResult.top) + window.pageYOffset,
            left: Math.round(leftResult.left) + window.pageXOffset,
            finalPlacement: finalPlacement,
            finalAlign: finalAlign,
            arrowOffset: arrowOffset,
        });
    };
    RawPopover.prototype.handleResize = function () {
        this.updatePosition();
    };
    RawPopover.prototype.handleScroll = function () {
        this.updatePosition();
    };
    RawPopover.defaultProps = {
        align: PopoverAlign.Left,
        placement: PopoverPlacement.Bottom,
    };
    tslib_1.__decorate([
        throttle_1.THROTTLE(100),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", []),
        tslib_1.__metadata("design:returntype", void 0)
    ], RawPopover.prototype, "handleResize", null);
    tslib_1.__decorate([
        throttle_1.THROTTLE(100),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", []),
        tslib_1.__metadata("design:returntype", void 0)
    ], RawPopover.prototype, "handleScroll", null);
    RawPopover = tslib_1.__decorate([
        pure_1.PURE,
        tslib_1.__metadata("design:paramtypes", [Object])
    ], RawPopover);
    return RawPopover;
}(React.Component));
exports.Popover = withTheme_1.withTheme(exports.POPOVER)(RawPopover);
function getArrowStyle(placement, align, offset) {
    switch (placement) {
        case PopoverPlacement.Top: //fallthrough
        case PopoverPlacement.Bottom: {
            switch (align) {
                case PopoverAlign.Left: {
                    return prefix_1.default({
                        left: offset,
                        transform: 'translateX(-50%)',
                    });
                }
                case PopoverAlign.Center: {
                    return prefix_1.default({
                        left: '50%',
                        transform: 'translateX(-50%)',
                    });
                }
                case PopoverAlign.Right: {
                    return prefix_1.default({
                        right: offset,
                        transform: 'translateX(50%)',
                    });
                }
            }
            break;
        }
        case PopoverPlacement.Left: //fallthrough
        case PopoverPlacement.Right: {
            switch (align) {
                case PopoverAlign.Top: {
                    return prefix_1.default({
                        top: offset,
                        transform: 'translateY(-50%)',
                    });
                }
                case PopoverAlign.Middle: {
                    return prefix_1.default({
                        top: '50%',
                        transform: 'translateY(-50%)',
                    });
                }
                case PopoverAlign.Bottom: {
                    return prefix_1.default({
                        bottom: offset,
                        transform: 'translateY(50%)',
                    });
                }
            }
        }
    }
    return undefined;
}
function movePopoverVertically(placement, align, anchorTop, anchorBottom, popoverHeight, checkBounds) {
    if (checkBounds === void 0) { checkBounds = false; }
    switch (placement) {
        case PopoverPlacement.Top: {
            var top_1 = anchorTop - popoverHeight;
            if (checkBounds && top_1 < 0) {
                return movePopoverVertically(PopoverPlacement.Bottom, align, anchorTop, anchorBottom, popoverHeight);
            }
            return {
                top: top_1,
                placement: placement,
                align: align,
            };
        }
        case PopoverPlacement.Bottom: {
            var top_2 = anchorBottom;
            if (checkBounds && top_2 + popoverHeight > window.innerHeight) {
                return movePopoverVertically(PopoverPlacement.Top, align, anchorTop, anchorBottom, popoverHeight);
            }
            return {
                top: top_2,
                placement: placement,
                align: align,
            };
        }
    }
    switch (align) {
        case PopoverAlign.Top: {
            var top_3 = anchorTop;
            if (checkBounds && top_3 + popoverHeight > window.innerHeight) {
                var resultForMiddle = movePopoverVertically(placement, PopoverAlign.Middle, anchorTop, anchorBottom, popoverHeight);
                if (resultForMiddle && resultForMiddle.top + popoverHeight > window.innerHeight) {
                    return movePopoverVertically(placement, PopoverAlign.Bottom, anchorTop, anchorBottom, popoverHeight);
                }
                return resultForMiddle;
            }
            return {
                top: top_3,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Middle: {
            var top_4 = anchorTop + (anchorBottom - anchorTop) / 2 - popoverHeight / 2;
            if (checkBounds) {
                if (top_4 < 0) {
                    return movePopoverVertically(placement, PopoverAlign.Top, anchorTop, anchorBottom, popoverHeight);
                }
                else if (top_4 + popoverHeight > window.innerHeight) {
                    return movePopoverVertically(placement, PopoverAlign.Bottom, anchorTop, anchorBottom, popoverHeight);
                }
            }
            return {
                top: top_4,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Bottom: {
            var top_5 = anchorBottom - popoverHeight;
            if (checkBounds && top_5 < 0) {
                var resultForMiddle = movePopoverVertically(placement, PopoverAlign.Middle, anchorTop, anchorBottom, popoverHeight);
                if (resultForMiddle && resultForMiddle.top < 0) {
                    return movePopoverVertically(placement, PopoverAlign.Top, anchorTop, anchorBottom, popoverHeight);
                }
                return resultForMiddle;
            }
            return {
                top: top_5,
                placement: placement,
                align: align,
            };
        }
    }
    return undefined;
}
function movePopoverHorizontally(placement, align, anchorLeft, anchorRight, popoverWidth, checkBounds) {
    if (checkBounds === void 0) { checkBounds = false; }
    switch (placement) {
        case PopoverPlacement.Left: {
            var left = anchorLeft - popoverWidth;
            if (checkBounds && left < 0) {
                return movePopoverHorizontally(PopoverPlacement.Right, align, anchorLeft, anchorRight, popoverWidth);
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
        case PopoverPlacement.Right: {
            var left = anchorRight;
            if (checkBounds && left + popoverWidth > window.innerWidth) {
                return movePopoverHorizontally(PopoverPlacement.Left, align, anchorLeft, anchorRight, popoverWidth);
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
    }
    switch (align) {
        case PopoverAlign.Left: {
            var left = anchorLeft;
            if (checkBounds && left + popoverWidth > window.innerWidth) {
                var resultForCenter = movePopoverHorizontally(placement, PopoverAlign.Center, anchorLeft, anchorRight, popoverWidth);
                if (resultForCenter && resultForCenter.left + popoverWidth > window.innerWidth) {
                    return movePopoverHorizontally(placement, PopoverAlign.Right, anchorLeft, anchorRight, popoverWidth);
                }
                return resultForCenter;
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Center: {
            var left = anchorLeft + (anchorRight - anchorLeft) / 2 - popoverWidth / 2;
            if (checkBounds) {
                if (left < 0) {
                    return movePopoverHorizontally(placement, PopoverAlign.Left, anchorLeft, anchorRight, popoverWidth);
                }
                else if (left + popoverWidth > window.innerWidth) {
                    return movePopoverHorizontally(placement, PopoverAlign.Right, anchorLeft, anchorRight, popoverWidth);
                }
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
        case PopoverAlign.Right: {
            var left = anchorRight - popoverWidth;
            if (checkBounds && left < 0) {
                var resultForCenter = movePopoverHorizontally(placement, PopoverAlign.Center, anchorLeft, anchorRight, popoverWidth);
                if (resultForCenter && resultForCenter.left < 0) {
                    return movePopoverHorizontally(placement, PopoverAlign.Left, anchorLeft, anchorRight, popoverWidth);
                }
                return resultForCenter;
            }
            return {
                left: left,
                placement: placement,
                align: align,
            };
        }
    }
    return undefined;
}
function getPlacementModifier(placement) {
    switch (placement) {
        case PopoverPlacement.Bottom: {
            return 'placementBottom';
        }
        case PopoverPlacement.Top: {
            return 'placementTop';
        }
        case PopoverPlacement.Left: {
            return 'placementLeft';
        }
        case PopoverPlacement.Right: {
            return 'placementRight';
        }
    }
}
//# sourceMappingURL=Popover.js.map